package dev.shadowsoffire.apotheosis.adventure.socket.gem.cutting;

import java.util.ArrayList;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.Container;
import net.minecraftforge.items.IItemHandlerModifiable;
import net.minecraftforge.items.wrapper.RecipeWrapper;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemInstance;
import java.util.Iterator;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import net.minecraft.server.level.ServerPlayer;
import dev.shadowsoffire.apotheosis.advancements.AdvancementTriggers;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import java.util.function.Predicate;
import net.minecraft.world.inventory.Slot;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.entity.player.Inventory;
import dev.shadowsoffire.placebo.cap.InternalItemHandler;
import net.minecraft.world.inventory.ContainerLevelAccess;
import net.minecraft.world.entity.player.Player;
import java.util.List;
import dev.shadowsoffire.placebo.menu.PlaceboContainerMenu;

public class GemCuttingMenu extends PlaceboContainerMenu
{
    public static final int NEXT_MAT_COST = 1;
    public static final int STD_MAT_COST = 3;
    public static final int PREV_MAT_COST = 9;
    public static final List<GemCuttingRecipe> RECIPES;
    protected final Player player;
    protected final ContainerLevelAccess access;
    protected final InternalItemHandler inv;
    
    public GemCuttingMenu(final int id, final Inventory playerInv) {
        this(id, playerInv, ContainerLevelAccess.f_39287_);
    }
    
    public GemCuttingMenu(final int id, final Inventory playerInv, final ContainerLevelAccess access) {
        super((MenuType)Adventure.Menus.GEM_CUTTING.get(), id, playerInv);
        this.inv = new InternalItemHandler(4) {
            public int getSlotLimit(final int slot) {
                return (slot == 0) ? 1 : super.getSlotLimit(slot);
            }
        };
        this.player = playerInv.f_35978_;
        this.access = access;
        this.m_38897_((Slot)new PlaceboContainerMenu.UpdatingSlot((PlaceboContainerMenu)this, this.inv, 0, 62, 45, stack -> GemItem.getGem(stack).isBound()));
        this.m_38897_((Slot)new PlaceboContainerMenu.UpdatingSlot((PlaceboContainerMenu)this, this.inv, 1, 90, 64, stack -> stack.m_41720_() == Adventure.Items.GEM_DUST.get()));
        this.m_38897_((Slot)new PlaceboContainerMenu.UpdatingSlot((PlaceboContainerMenu)this, this.inv, 2, 33, 64, (Predicate)this::matchesMainGem));
        this.m_38897_((Slot)new PlaceboContainerMenu.UpdatingSlot((PlaceboContainerMenu)this, this.inv, 3, 62, 12, (Predicate)this::isValidMaterial));
        this.addPlayerSlots(playerInv, 8, 98);
        this.mover.registerRule((stack, slot) -> slot >= this.playerInvStart && this.inv.getStackInSlot(0).m_41619_() && isValidMainGem(stack), 0, 1);
        this.mover.registerRule((stack, slot) -> slot >= this.playerInvStart && stack.m_41720_() == Adventure.Items.GEM_DUST.get(), 1, 2);
        this.mover.registerRule((stack, slot) -> slot >= this.playerInvStart && this.matchesMainGem(stack), 2, 3);
        this.mover.registerRule((stack, slot) -> slot >= this.playerInvStart && this.isValidMaterial(stack), 3, 4);
        this.mover.registerRule((stack, slot) -> slot < this.playerInvStart, this.playerInvStart, this.hotbarStart + 9);
        this.registerInvShuffleRules();
    }
    
    public boolean m_6366_(final Player player, final int id) {
        if (id == 0) {
            final ItemStack gem = this.inv.getStackInSlot(0);
            final ItemStack left = this.inv.getStackInSlot(1);
            final ItemStack bot = this.inv.getStackInSlot(2);
            final ItemStack right = this.inv.getStackInSlot(3);
            for (final GemCuttingRecipe r : GemCuttingMenu.RECIPES) {
                if (r.matches(gem, left, bot, right)) {
                    final ItemStack out = r.getResult(gem, left, bot, right);
                    r.decrementInputs(gem, left, bot, right);
                    this.inv.setStackInSlot(0, out);
                    this.level.m_5594_(player, player.m_20183_(), SoundEvents.f_144242_, SoundSource.BLOCKS, 1.0f, 1.5f + 0.35f * (1.0f - 2.0f * this.level.f_46441_.m_188501_()));
                    AdvancementTriggers.GEM_CUT.trigger((ServerPlayer)player, out, AffixHelper.getRarity(out).getId());
                    return true;
                }
            }
        }
        return false;
    }
    
    public static boolean isValidMainGem(final ItemStack stack) {
        final GemInstance inst = GemInstance.unsocketed(stack);
        return inst.isValidUnsocketed() && !inst.isMaxRarity();
    }
    
    protected boolean isValidMaterial(final ItemStack stack) {
        final GemInstance mainGem = GemInstance.unsocketed(this.inv.getStackInSlot(0));
        if (!mainGem.isValidUnsocketed()) {
            return false;
        }
        final DynamicHolder<LootRarity> rarity = (DynamicHolder<LootRarity>)RarityRegistry.getMaterialRarity(stack.m_41720_());
        return rarity.isBound() && Math.abs(((LootRarity)rarity.get()).ordinal() - ((LootRarity)mainGem.rarity().get()).ordinal()) <= 1;
    }
    
    protected boolean matchesMainGem(final ItemStack stack) {
        final GemInstance gem = GemInstance.unsocketed(stack);
        final GemInstance mainGem = GemInstance.unsocketed(this.inv.getStackInSlot(0));
        return gem.isValidUnsocketed() && mainGem.isValidUnsocketed() && gem.gem() == mainGem.gem() && gem.rarity() == mainGem.rarity();
    }
    
    public boolean m_6875_(final Player pPlayer) {
        return (boolean)this.access.m_39299_((level, pos) -> level.m_8055_(pos).m_60734_() == Adventure.Blocks.GEM_CUTTING_TABLE.get(), (Object)true);
    }
    
    public void m_6877_(final Player pPlayer) {
        super.m_6877_(pPlayer);
        this.access.m_39292_((level, pos) -> this.m_150411_(pPlayer, (Container)new RecipeWrapper((IItemHandlerModifiable)this.inv)));
    }
    
    public static int getDustCost(final LootRarity gemRarity) {
        return 1 + gemRarity.ordinal() * 2;
    }
    
    static {
        (RECIPES = new ArrayList<GemCuttingRecipe>()).add(new RarityUpgrade());
    }
    
    public static class RarityUpgrade implements GemCuttingRecipe
    {
        @Override
        public boolean matches(final ItemStack gem, final ItemStack left, final ItemStack bot, final ItemStack right) {
            final GemInstance g = GemInstance.unsocketed(gem);
            final GemInstance g2 = GemInstance.unsocketed(bot);
            if (!g.isValidUnsocketed() || !g2.isValidUnsocketed() || g.gem() != g2.gem() || g.rarity() != g2.rarity()) {
                return false;
            }
            if (g.isMaxRarity()) {
                return false;
            }
            if (left.m_41720_() != Adventure.Items.GEM_DUST.get() || left.m_41613_() < GemCuttingMenu.getDustCost((LootRarity)g.rarity().get())) {
                return false;
            }
            if (!RarityRegistry.isMaterial(right.m_41720_())) {
                return false;
            }
            final DynamicHolder<LootRarity> matRarity = (DynamicHolder<LootRarity>)RarityRegistry.getMaterialRarity(right.m_41720_());
            final DynamicHolder<LootRarity> gemRarity = (DynamicHolder<LootRarity>)g.rarity();
            if (matRarity == gemRarity) {
                return right.m_41613_() >= 3;
            }
            if (matRarity == RarityRegistry.next((DynamicHolder)gemRarity)) {
                return right.m_41613_() >= 1;
            }
            return matRarity == RarityRegistry.prev((DynamicHolder)gemRarity) && right.m_41613_() >= 9;
        }
        
        @Override
        public ItemStack getResult(final ItemStack gem, final ItemStack left, final ItemStack bot, final ItemStack right) {
            final ItemStack out = gem.m_41777_();
            AffixHelper.setRarity(out, (LootRarity)RarityRegistry.next(AffixHelper.getRarity(out)).get());
            return out;
        }
        
        @Override
        public void decrementInputs(final ItemStack gem, final ItemStack left, final ItemStack bot, final ItemStack right) {
            final DynamicHolder<LootRarity> matRarity = (DynamicHolder<LootRarity>)RarityRegistry.getMaterialRarity(right.m_41720_());
            final DynamicHolder<LootRarity> gemRarity = (DynamicHolder<LootRarity>)GemInstance.unsocketed(gem).rarity();
            gem.m_41774_(1);
            left.m_41774_(GemCuttingMenu.getDustCost((LootRarity)gemRarity.get()));
            bot.m_41774_(1);
            right.m_41774_((matRarity == gemRarity) ? 3 : ((matRarity == RarityRegistry.next((DynamicHolder)gemRarity)) ? 1 : 9));
        }
    }
    
    public interface GemCuttingRecipe
    {
        boolean matches(final ItemStack p0, final ItemStack p1, final ItemStack p2, final ItemStack p3);
        
        ItemStack getResult(final ItemStack p0, final ItemStack p1, final ItemStack p2, final ItemStack p3);
        
        void decrementInputs(final ItemStack p0, final ItemStack p1, final ItemStack p2, final ItemStack p3);
    }
}
